/* 9-131.c */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "emps.h"
/* This program simulates a string using a linked list of characters.
   The operations that may be performed on the string are:
   insert a character
   remove a character
   print the all the characters.
*/

/* Create a type definition describing a node in the linked list.
   Each node contains a character value, and the location of the 
   next character in the list. */

typedef struct alpha_node  
{
   char val;
   struct alpha_node *next;
} alpha;

/* the legal operations that may be performed on the linked list */
void insert_node (alpha **head, alpha **tail, alpha temp);
void print_nodes (const alpha *head);
void remove_node (alpha **head, alpha **tail, char c);

void main (void) 
{
   /* initialize the pointers */
   alpha *head = NULL, 
         *tail = NULL; 
   alpha  tmp;
   int c;

   /* insert the values into the list */
   for (c='A'; c<'F'; c++) 
   { 
      /* insert the char value into the structure */
      tmp.val = c;

      /* insert the structure into the list */
      insert_node (&head, &tail, tmp); 
   }

   /* display the list */
   print_nodes (head);

   printf ("Which node should be deleted (q to quit) : ");
   
   /* as long as there are nodes, and user wants to continue */
   while ((head != NULL) && ((c = toupper(getchar())) != 'Q'))
   {
      fflush (stdin);

      /* remove the requested node */
      remove_node (&head, &tail, c);

      /* print the remaining list */
      print_nodes (head);
   }
}

void insert_node (alpha **head, alpha **tail, alpha temp) 
{
   /* initialize the new node pointer */
   alpha *newn = NULL;

   /* allocate the new node */
   if (NULL == (newn = (alpha *)malloc (sizeof (alpha)))) 
   {
      perror ("insert_node");
      exit (EXIT_FAILURE); 
   }

   /* assign the contents to the newly allocated space */
   *newn = temp;

   /* always initialize your pointers */
   newn->next = NULL;

   /* Will this be the first node in the list? */
   if (*head == NULL)
      /* initialize the pointers */
      *head = *tail = newn; 
   else 
   {
      /* append the new node onto the end of the list */
      (*tail)->next = newn; 
      *tail = (*tail)->next;
   }
}

void print_nodes (const alpha *head)
{
   /* starting with the first node in the list */
   const alpha *tmp_node = head;

   /* traverse the list, printing values until you hit the NULL sentinel */
   while (tmp_node != NULL) 
   {
      printf ("%c -> ",tmp_node->val);
      /* advance the pointer to the next node in the list */
      tmp_node = tmp_node->next; 
   }
   /* at NULL sentinel */
   printf ("NULL\n");
}

void remove_node (alpha **head, alpha **tail, char c)
{
   /* create two temp pointers:
         1 - node to remove, and
         2 - previous node, to reconnect to removed node's neighbor 
   */ 
   alpha *tmp  = *head;
   alpha *prev = *head;

   /* check if you are removing the first (head) node */
   if ((tmp != NULL) && (tmp->val == c))
   {
      /* reassign the head to the next node in the list */
      *head = (*head)->next;

      /* release the removed node's space back to the system */
      free(tmp);
      return;
   }

   /* traverse the list, looking for the value to be removed */
   while ((tmp != NULL) && (tmp->val != c))
   {
      /* trail behind the tmp pointer, for later reconnection */
      prev = tmp;
      tmp  = tmp->next;
   }

   /* now you've either found the node to remove, or you've exhausted
      the list 
   */
   if (NULL == tmp)
      printf ("Character %c not found in the list.\n", c);
   else
   {   
      /* found the node, reassign previous pointer to next */
      prev->next = tmp->next;
      free (tmp);  
   }
   if (NULL == prev->next)
      /* you deleted the last node in the list, so reassign the tail */
      *tail = prev;
}
